<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.


/**
 * This is form to display the modules for course tracking reports
 *
 * @package   bcln_course_tracking
 * @copyright 2018 BCLN
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die;

require_once($CFG->libdir.'/formslib.php');
require_once(dirname(__FILE__) . '/lib.php');


/**
 * This is form to display the dates for setting dates
 *
 * @copyright 2018 BCLN
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class report_editdates_form extends moodleform {
    /**
     * @see lib/moodleform#definition()
     */
    public function definition() {
        global $CFG, $COURSE, $DB, $USER;
        $mform = $this->_form;

        $modinfo       = $this->_customdata['modinfo'];
        $course        = $this->_customdata['course'];
        $instanceid    = $this->_customdata['instanceid'];
        
        $coursehasavailability = !empty($CFG->enableavailability);
        $coursehascompletion   = !empty($CFG->enablecompletion) && !empty($course->enablecompletion);

        // Context instance of the course.
        $coursecontext = context_course::instance($course->id);
       // Add action button to the top of the form.
        $addactionbuttons = false;
        $this->add_action_buttons();

        // Var to count the number of elements in the course/sections.
        // It will be used to decide whether to show save action button
        // at the bottom of the form page.
        $uid = $USER->id;
        $block = $DB->get_record('block_instances', array('id' => $instanceid), '*', MUST_EXIST);
        $config = unserialize(base64_decode($block->configdata));
        $activities = block_individual_progress_get_activities($course->id, $config);
        foreach ($activities as $acts) {
            $stractivityname = html_writer::tag('strong' , $acts['name']);
            $mform->addElement('static', 'modname' . $acts['id'], $stractivityname);
            $elname = 'deadline-'.$acts['id'];
            $mform->addElement('date_selector',$elname, get_string('deadline', 'block_individual_progress'));
         // check in the database to see if the values are there
            if ($DB->record_exists('individual_progress', 
                     array('course' => $course->id, 'userid' => $uid, 'coursemoduleid' => $acts['id']))) {
               $result = $DB->get_record('individual_progress',
                      array('course' => $course->id, 'userid' => $uid, 'coursemoduleid' => $acts['id']));
               if ($result->deadline > '0') {
                   $elname = 'deadline-'.$acts['id'];
                   $mform->setDefault($elname,$result->deadline);
               }
            }   
        }        

        $this->add_action_buttons();
        
    }

    public function validation($data, $files) {
        global $CFG;
        $errors = parent::validation($data, $files);

        $modinfo = $this->_customdata['modinfo'];
        $course = $this->_customdata['course'];
        $coursecontext = context_course::instance($course->id);

        $moddatesettings = array();
        $forceddatesettings = array();
        foreach ($data as $key => $value) {
            if ($key == "coursestartdate") {
                continue;
            }

            $cmsettings = explode('_', $key);
            // The array should have 4 keys.
            if (count($cmsettings) != 4) {
                continue;
            }

            // Ignore 0th position, it will be 'date'
            // 1st position should be the mod type
            // 2nd will be the id of module
            // 3rd will be property of module
            // ensure that the name is proper.
            if (isset($cmsettings['1']) && isset($cmsettings['2']) && isset($cmsettings['3'])) {
                // Check if its mod date settings.
                if ($cmsettings['1'] == 'mod') {
                    // Check if config date settings are forced
                    // and this is one of the forced date setting.
                    if (($CFG->enableavailability || $CFG->enablecompletion )
                            && in_array($cmsettings['3'], array('completionexpected', 'availablefrom', 'availableuntil'))) {
                        $forceddatesettings[$cmsettings['2']][$cmsettings['3']] = $value;
                    } else {
                        // It is module date setting.
                        $moddatesettings[$cmsettings['2']][$cmsettings['3']] = $value;
                    }
                }
            }
        }

        $cms = $modinfo->get_cms();

        // Validating forced date settings.
        foreach ($forceddatesettings as $modid => $datesettings) {
            // Course module object.
            $cm = $cms[$modid];
            $moderrors = array();
            if (isset($datesettings['availablefrom']) && isset($datesettings['availableuntil'])
                    && $datesettings['availablefrom'] != 0 && $datesettings['availableuntil'] != 0
                    && $datesettings['availablefrom'] > $datesettings['availableuntil'] ) {
                $errors['date_mod_'.$modid.'_availableuntil'] =
                    get_string('badavailabledates', 'condition');
            }
        }

        // Validating mod date settings.
        foreach ($moddatesettings as $modid => $datesettings) {
            // Course module object.
            $cm = $cms[$modid];
            $moderrors = array();

            if ($mod = report_editdates_mod_date_extractor::make($cm->modname, $course)) {
                $moderrors = $mod->validate_dates($cm, $datesettings);
                if (!empty($moderrors)) {
                    foreach ($moderrors as $errorfield => $errorstr) {
                        $errors['date_mod_'.$modid.'_'.$errorfield] = $errorstr;
                    }
                }
            }
        }

        return $errors;
    }
}
